@@ -10,26 +10,37 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import copy
-
 from oslo_config import cfg
 
 from oslo_log import versionutils
 
 _DEFAULT_LOG_DATE_FORMAT = "%Y-%m-%d %H:%M:%S"
 
+DEFAULT_LOG_LEVELS = ['amqp=WARN', 'amqplib=WARN', 'boto=WARN',
+                      'qpid=WARN', 'sqlalchemy=WARN', 'suds=INFO',
+                      'oslo.messaging=INFO', 'iso8601=WARN',
+                      'requests.packages.urllib3.connectionpool=WARN',
+                      'urllib3.connectionpool=WARN', 'websocket=WARN',
+                      'requests.packages.urllib3.util.retry=WARN',
+                      'urllib3.util.retry=WARN',
+                      'keystonemiddleware=WARN', 'routes.middleware=WARN',
+                      'stevedore=WARN', 'taskflow=WARN',
+                      'keystoneauth=WARN', 'oslo.cache=INFO',
+                      'dogpile.core.dogpile=INFO']
+
+_IGNORE_MESSAGE = "This option is ignored if log_config_append is set."
 
 common_cli_opts = [
     cfg.BoolOpt('debug',
                 short='d',
                 default=False,
-                help='Print debugging output (set logging level to '
-                     'DEBUG instead of default INFO level).'),
+                help='If set to true, the logging level will be set to '
+                     'DEBUG instead of the default INFO level.'),
     cfg.BoolOpt('verbose',
                 short='v',
                 default=True,
-                help='If set to false, will disable INFO logging level, '
-                     'making WARNING the default.',
+                help='If set to false, the logging level will be set to '
+                     'WARNING instead of the default INFO level.',
                 deprecated_for_removal=True),
 ]
 
@@ -40,63 +51,56 @@ logging_cli_opts = [
                help='The name of a logging configuration file. This file '
                     'is appended to any existing logging configuration '
                     'files. For details about logging configuration files, '
-                    'see the Python logging module documentation.'),
-    cfg.StrOpt('log-format',
-               metavar='FORMAT',
-               help='DEPRECATED. '
-                    'A logging.Formatter log message format string which may '
-                    'use any of the available logging.LogRecord attributes. '
-                    'This option is deprecated.  Please use '
-                    'logging_context_format_string and '
-                    'logging_default_format_string instead.'),
+                    'see the Python logging module documentation. Note that '
+                    'when logging configuration files are used then all '
+                    'logging configuration is set in the configuration file '
+                    'and other logging configuration options are ignored '
+                    '(for example, logging_context_format_string).'),
     cfg.StrOpt('log-date-format',
                default=_DEFAULT_LOG_DATE_FORMAT,
                metavar='DATE_FORMAT',
-               help='Format string for %%(asctime)s in log records. '
-                    'Default: %(default)s .'),
+               help='Defines the format string for %%(asctime)s in log '
+                    'records. Default: %(default)s . '
+                    + _IGNORE_MESSAGE),
     cfg.StrOpt('log-file',
                metavar='PATH',
                deprecated_name='logfile',
-               help='(Optional) Name of log file to output to. '
-                    'If no default is set, logging will go to stdout.'),
+               help='(Optional) Name of log file to send logging output to. '
+                    'If no default is set, logging will go to stderr as '
+                    'defined by use_stderr. '
+                    + _IGNORE_MESSAGE),
     cfg.StrOpt('log-dir',
                deprecated_name='logdir',
-               help='(Optional) The base directory used for relative '
-                    '--log-file paths.'),
+               help='(Optional) The base directory used for relative log_file '
+                    ' paths. '
+                    + _IGNORE_MESSAGE),
+    cfg.BoolOpt('watch-log-file',
+                default=False,
+                help='Uses logging handler designed to watch file '
+                     'system. When log file is moved or removed this handler '
+                     'will open a new log file with specified path '
+                     'instantaneously. It makes sense only if log_file option '
+                     'is specified and Linux platform is used. '
+                     + _IGNORE_MESSAGE),
     cfg.BoolOpt('use-syslog',
                 default=False,
                 help='Use syslog for logging. '
                      'Existing syslog format is DEPRECATED '
-                     'and will be changed later to honor RFC5424.'),
-    cfg.BoolOpt('use-syslog-rfc-format',
-                default=True,
-                deprecated_for_removal=True,
-                help='(Optional) Enables or disables syslog rfc5424 format '
-                     'for logging. If enabled, prefixes the MSG part of the '
-                     'syslog message with APP-NAME (RFC5424). The '
-                     'format without the APP-NAME is deprecated in Kilo, '
-                     'and will be removed in Mitaka, along with this option.'),
+                     'and will be changed later to honor RFC5424. '
+                     + _IGNORE_MESSAGE),
     cfg.StrOpt('syslog-log-facility',
                default='LOG_USER',
-               help='Syslog facility to receive log lines.')
+               help='Syslog facility to receive log lines. '
+                    + _IGNORE_MESSAGE),
 ]
 
 generic_log_opts = [
     cfg.BoolOpt('use_stderr',
                 default=True,
-                help='Log output to standard error.')
+                help='Log output to standard error. '
+                     + _IGNORE_MESSAGE),
 ]
 
-DEFAULT_LOG_LEVELS = ['amqp=WARN', 'amqplib=WARN', 'boto=WARN',
-                      'qpid=WARN', 'sqlalchemy=WARN', 'suds=INFO',
-                      'oslo.messaging=INFO', 'iso8601=WARN',
-                      'requests.packages.urllib3.connectionpool=WARN',
-                      'urllib3.connectionpool=WARN', 'websocket=WARN',
-                      'requests.packages.urllib3.util.retry=WARN',
-                      'urllib3.util.retry=WARN',
-                      "keystonemiddleware=WARN", "routes.middleware=WARN",
-                      "stevedore=WARN", "taskflow=WARN"]
-
 log_opts = [
     cfg.StrOpt('logging_context_format_string',
                default='%(asctime)s.%(msecs)03d %(process)d %(levelname)s '
@@ -106,17 +110,25 @@ log_opts = [
     cfg.StrOpt('logging_default_format_string',
                default='%(asctime)s.%(msecs)03d %(process)d %(levelname)s '
                        '%(name)s [-] %(instance)s%(message)s',
-               help='Format string to use for log messages without context.'),
+               help='Format string to use for log messages when context is '
+                    'undefined.'),
     cfg.StrOpt('logging_debug_format_suffix',
                default='%(funcName)s %(pathname)s:%(lineno)d',
-               help='Data to append to log format when level is DEBUG.'),
+               help='Additional data to append to log message when logging '
+                    'level for the message is DEBUG.'),
     cfg.StrOpt('logging_exception_prefix',
                default='%(asctime)s.%(msecs)03d %(process)d ERROR %(name)s '
                '%(instance)s',
                help='Prefix each line of exception output with this format.'),
+    cfg.StrOpt('logging_user_identity_format',
+               default='%(user)s %(tenant)s '
+                       '%(domain)s %(user_domain)s %(project_domain)s',
+               help='Defines the format string for %(user_identity)s that '
+                    'is used in logging_context_format_string.'),
     cfg.ListOpt('default_log_levels',
                 default=DEFAULT_LOG_LEVELS,
-                help='List of logger=LEVEL pairs.'),
+                help='List of package logging levels in logger=LEVEL pairs. '
+                     + _IGNORE_MESSAGE),
     cfg.BoolOpt('publish_errors',
                 default=False,
                 help='Enables or disables publication of error events.'),
@@ -147,10 +159,11 @@ def list_opts():
     config files.
 
     The purpose of this is to allow tools like the Oslo sample config file
-    generator to discover the options exposed to users by this library.
+    generator (oslo-config-generator) to discover the options exposed to users
+    by this library.
 
     :returns: a list of (group_name, opts) tuples
     """
-    return [(None, copy.deepcopy(common_cli_opts + logging_cli_opts +
-                                 generic_log_opts + log_opts +
-                                 versionutils.deprecated_opts))]
+    return [(None, (common_cli_opts + logging_cli_opts +
+                    generic_log_opts + log_opts +
+                    versionutils.deprecated_opts))]
