@@ -10,13 +10,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import datetime
 import itertools
 import logging
 import logging.config
 import logging.handlers
+import socket
 import sys
 import traceback
 
+from dateutil import tz
 import six
 from six import moves
 
@@ -61,11 +64,20 @@ def _update_record_with_context(record):
     return context
 
 
+class _ReplaceFalseValue(dict):
+    def __getitem__(self, key):
+        return dict.get(self, key, None) or '-'
+
+
 class JSONFormatter(logging.Formatter):
     def __init__(self, fmt=None, datefmt=None):
         # NOTE(jkoelker) we ignore the fmt argument, but its still there
         #                since logging.config.fileConfig passes it.
         self.datefmt = datefmt
+        try:
+            self.hostname = socket.gethostname()
+        except socket.error:
+            self.hostname = None
 
     def formatException(self, ei, strip_newlines=True):
         lines = traceback.format_exception(*ei)
@@ -96,7 +108,8 @@ class JSONFormatter(logging.Formatter):
                    'thread_name': record.threadName,
                    'process_name': record.processName,
                    'process': record.process,
-                   'traceback': None}
+                   'traceback': None,
+                   'hostname': self.hostname}
 
         # Build the extra values that were given to us, including
         # the context.
@@ -130,11 +143,18 @@ class ContextFormatter(logging.Formatter):
     logging_debug_format_suffix to append extra formatting if the log level is
     debug.
 
-    For information about what variables are available for the formatter see:
+    The standard variables available to the formatter are listed at:
     http://docs.python.org/library/logging.html#formatter
 
-    If available, uses the context value stored in TLS - local.store.context
+    In addition to the standard variables, one custom variable is
+    available to both formatting string: `isotime` produces a
+    timestamp in ISO8601 format, suitable for producing
+    RFC5424-compliant log messages.
 
+    Furthermore, logging_context_format_string has access to all of
+    the data in a dict representation of the context.
+
+    If available, uses the context value stored in TLS - local.store.context
     """
 
     def __init__(self, *args, **kwargs):
@@ -216,6 +236,15 @@ class ContextFormatter(logging.Formatter):
             if key not in record.__dict__:
                 record.__dict__[key] = ''
 
+        # Set the "user_identity" value of "logging_context_format_string"
+        # by using "logging_user_identity_format" and
+        # "to_dict()" of oslo.context.
+        if context:
+            record.user_identity = (
+                self.conf.logging_user_identity_format %
+                _ReplaceFalseValue(context.__dict__)
+            )
+
         if record.__dict__.get('request_id'):
             fmt = self.conf.logging_context_format_string
         else:
@@ -225,6 +254,8 @@ class ContextFormatter(logging.Formatter):
                 self.conf.logging_debug_format_suffix):
             fmt += " " + self.conf.logging_debug_format_suffix
 
+        self._compute_iso_time(record)
+
         if sys.version_info < (3, 2):
             self._fmt = fmt
         else:
@@ -249,9 +280,26 @@ class ContextFormatter(logging.Formatter):
         if self.conf.logging_exception_prefix.find('%(asctime)') != -1:
             record.asctime = self.formatTime(record, self.datefmt)
 
+        self._compute_iso_time(record)
+
         formatted_lines = []
         for line in lines:
             pl = self.conf.logging_exception_prefix % record.__dict__
             fl = '%s%s' % (pl, line)
             formatted_lines.append(fl)
         return '\n'.join(formatted_lines)
+
+    def _compute_iso_time(self, record):
+        # set iso8601 timestamp
+        localtz = tz.tzlocal()
+        record.isotime = datetime.datetime.fromtimestamp(
+            record.created).replace(tzinfo=localtz).isoformat()
+        if record.created == int(record.created):
+            # NOTE(stpierre): when the timestamp includes no
+            # microseconds -- e.g., 1450274066.000000 -- then the
+            # microseconds aren't included in the isoformat() time. As
+            # a result, in literally one in a million cases
+            # isoformat() looks different. This adds microseconds when
+            # that happens.
+            record.isotime = "%s.000000%s" % (record.isotime[:-6],
+                                              record.isotime[-6:])
