@@ -31,6 +31,7 @@ import logging
 import logging.config
 import logging.handlers
 import os
+import platform
 import sys
 try:
     import syslog
@@ -44,8 +45,6 @@ from oslo_utils import importutils
 import six
 from six import moves
 
-_PY26 = sys.version_info[0:2] == (2, 6)
-
 from oslo_log._i18n import _
 from oslo_log import _options
 from oslo_log import formatters
@@ -92,15 +91,6 @@ class BaseLoggerAdapter(logging.LoggerAdapter):
     def trace(self, msg, *args, **kwargs):
         self.log(TRACE, msg, *args, **kwargs)
 
-    def isEnabledFor(self, level):
-        if _PY26:
-            # This method was added in python 2.7 (and it does the exact
-            # same logic, so we need to do the exact same logic so that
-            # python 2.6 has this capability as well).
-            return self.logger.isEnabledFor(level)
-        else:
-            return super(BaseLoggerAdapter, self).isEnabledFor(level)
-
 
 def _ensure_unicode(msg):
     """Do our best to turn the input argument into a unicode object.
@@ -231,6 +221,13 @@ def _load_log_config(log_config_append):
 
 def register_options(conf):
     """Register the command line and configuration options used by oslo.log."""
+
+    # Sometimes logging occurs before logging is ready (e.g., oslo_config).
+    # To avoid "No handlers could be found," temporarily log to sys.stderr.
+    root_logger = logging.getLogger(None)
+    if not root_logger.handlers:
+        root_logger.addHandler(logging.StreamHandler())
+
     conf.register_cli_opts(_options.common_cli_opts)
     conf.register_cli_opts(_options.logging_cli_opts)
     conf.register_opts(_options.generic_log_opts)
@@ -306,12 +303,20 @@ def _find_facility(facility):
 
 def _setup_logging_from_conf(conf, project, version):
     log_root = getLogger(None).logger
-    for handler in log_root.handlers:
+
+    # Remove all handlers
+    for handler in list(log_root.handlers):
         log_root.removeHandler(handler)
 
     logpath = _get_log_file_path(conf)
     if logpath:
-        filelog = logging.handlers.WatchedFileHandler(logpath)
+        if conf.watch_log_file and platform.system() == 'Linux':
+            from oslo_log import watchers
+            file_handler = watchers.FastWatchedFileHandler
+        else:
+            file_handler = logging.handlers.WatchedFileHandler
+
+        filelog = file_handler(logpath)
         log_root.addHandler(filelog)
 
     if conf.use_stderr:
@@ -335,27 +340,15 @@ def _setup_logging_from_conf(conf, project, version):
         if syslog is None:
             raise RuntimeError("syslog is not available on this platform")
         facility = _find_facility(conf.syslog_log_facility)
-        # TODO(bogdando) use the format provided by RFCSysLogHandler after
-        # existing syslog format deprecation in J
-        syslog = handlers.OSSysLogHandler(
-            facility=facility,
-            use_syslog_rfc_format=conf.use_syslog_rfc_format)
-        log_root.addHandler(syslog)
+        syslog_handler = handlers.OSSysLogHandler(facility=facility)
+        log_root.addHandler(syslog_handler)
 
     datefmt = conf.log_date_format
     for handler in log_root.handlers:
-        # NOTE(alaski): CONF.log_format overrides everything currently.  This
-        # should be deprecated in favor of context aware formatting.
-        if conf.log_format:
-            handler.setFormatter(logging.Formatter(fmt=conf.log_format,
-                                                   datefmt=datefmt))
-            log_root.info('Deprecated: log_format is now deprecated and will '
-                          'be removed in the next release')
-        else:
-            handler.setFormatter(formatters.ContextFormatter(project=project,
-                                                             version=version,
-                                                             datefmt=datefmt,
-                                                             config=conf))
+        handler.setFormatter(formatters.ContextFormatter(project=project,
+                                                         version=version,
+                                                         datefmt=datefmt,
+                                                         config=conf))
 
     if conf.debug:
         log_root.setLevel(logging.DEBUG)
@@ -374,19 +367,12 @@ def _setup_logging_from_conf(conf, project, version):
             # defined at level 5, so to make that accessible, try to convert
             # this to a integer, and if not keep the original...
             numeric_level = int(level_name)
-        except ValueError:
+        except ValueError:  # nosec
             pass
-        # NOTE(AAzza) in python2.6 Logger.setLevel doesn't convert string name
-        # to integer code.
-        if sys.version_info < (2, 7):
-            if numeric_level is None:
-                numeric_level = logging.getLevelName(level_name)
+        if numeric_level is not None:
             logger.setLevel(numeric_level)
         else:
-            if numeric_level is not None:
-                logger.setLevel(numeric_level)
-            else:
-                logger.setLevel(level_name)
+            logger.setLevel(level_name)
 
 _loggers = {}
 
@@ -416,3 +402,13 @@ def getLogger(name=None, project='unknown', version='unknown'):
                                                 {'project': project,
                                                  'version': version})
     return _loggers[name]
+
+
+def get_default_log_levels():
+    """Return the Oslo Logging default log levels.
+
+    Returns a copy of the list so an application can change the value
+    and not affect the default value used in the log_opts configuration
+    setup.
+    """
+    return list(_options.DEFAULT_LOG_LEVELS)
