@@ -12,7 +12,12 @@
 
 import logging
 
+from oslo_log import versionutils
 
+
+@versionutils.deprecated(
+    versionutils.deprecated.LIBERTY,
+    remove_in=+1)
 class WritableLogger(object):
     """A thin wrapper that responds to `write` and logs."""
 
@@ -21,4 +26,4 @@ class WritableLogger(object):
         self.level = level
 
     def write(self, msg):
-        self.logger.log(self.level, msg.rstrip())
\ No newline at end of file
+        self.logger.log(self.level, msg.rstrip())
