@@ -14,15 +14,19 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import datetime
 import logging
 import os
+import platform
 import sys
 try:
     import syslog
 except ImportError:
     syslog = None
 import tempfile
+import time
 
+from dateutil import tz
 import mock
 from oslo_config import cfg
 from oslo_config import fixture as fixture_config  # noqa
@@ -41,7 +45,14 @@ from oslo_log import log
 
 
 def _fake_context():
-    return context.RequestContext(1, 1, overwrite=True)
+    ctxt = context.RequestContext(1, 1, overwrite=True)
+    ctxt.user = 'myuser'
+    ctxt.tenant = 'mytenant'
+    ctxt.domain = 'mydomain'
+    ctxt.project_domain = 'myprojectdomain'
+    ctxt.user_domain = 'myuserdomain'
+
+    return ctxt
 
 
 class CommonLoggerTestsMixIn(object):
@@ -117,6 +128,7 @@ class BaseTestCase(test_base.BaseTestCase):
         self.config = self.config_fixture.config
         self.CONF = self.config_fixture.conf
         log.register_options(self.CONF)
+        log.setup(self.CONF, 'base')
 
 
 class LogTestBase(BaseTestCase):
@@ -188,27 +200,15 @@ class LogHandlerTestCase(BaseTestCase):
 
 
 class SysLogHandlersTestCase(BaseTestCase):
-    """Test for standard and RFC compliant Syslog handlers."""
+    """Test the standard Syslog handler."""
     def setUp(self):
         super(SysLogHandlersTestCase, self).setUp()
         self.facility = logging.handlers.SysLogHandler.LOG_USER
-        self.rfclogger = handlers.RFCSysLogHandler(facility=self.facility)
-        self.rfclogger.binary_name = 'Foo_application'
         self.logger = logging.handlers.SysLogHandler(facility=self.facility)
-        self.logger.binary_name = 'Foo_application'
-
-    def test_rfc_format(self):
-        """Ensure syslog msg contains APP-NAME for RFC wrapped handler."""
-        logrecord = logging.LogRecord('name', 'WARN', '/tmp', 1,
-                                      'Message', None, None)
-        expected = logging.LogRecord('name', 'WARN', '/tmp', 1,
-                                     'Foo_application Message', None, None)
-        self.assertEqual(self.rfclogger.format(logrecord),
-                         expected.getMessage())
 
     def test_standard_format(self):
         """Ensure syslog msg isn't modified for standard handler."""
-        logrecord = logging.LogRecord('name', 'WARN', '/tmp', 1,
+        logrecord = logging.LogRecord('name', logging.WARNING, '/tmp', 1,
                                       'Message', None, None)
         expected = logrecord
         self.assertEqual(self.logger.format(logrecord),
@@ -217,7 +217,7 @@ class SysLogHandlersTestCase(BaseTestCase):
 
 @testtools.skipUnless(syslog, "syslog is not available")
 class OSSysLogHandlerTestCase(BaseTestCase):
-    def tests_handler(self):
+    def test_handler(self):
         handler = handlers.OSSysLogHandler()
         syslog.syslog = mock.Mock()
         handler.emit(
@@ -226,6 +226,15 @@ class OSSysLogHandlerTestCase(BaseTestCase):
                               None, None))
         self.assertTrue(syslog.syslog.called)
 
+    def test_syslog_binary_name(self):
+        # There is no way to test the actual output written to the
+        # syslog (e.g. /var/log/syslog) to confirm binary_name value
+        # is actually present
+        syslog.openlog = mock.Mock()
+        handlers.OSSysLogHandler()
+        syslog.openlog.assert_called_with(handlers._get_binary_name(),
+                                          0, syslog.LOG_USER)
+
     def test_find_facility(self):
         self.assertEqual(syslog.LOG_USER, log._find_facility("user"))
         self.assertEqual(syslog.LOG_LPR, log._find_facility("LPR"))
@@ -330,6 +339,15 @@ class JSONFormatterTestCase(LogTestBase):
         self.assertTrue(data['traceback'])
 
 
+def get_fake_datetime(retval):
+    class FakeDateTime(datetime.datetime):
+        @classmethod
+        def fromtimestamp(cls, timestamp):
+            return retval
+
+    return FakeDateTime
+
+
 class ContextFormatterTestCase(LogTestBase):
     def setUp(self):
         super(ContextFormatterTestCase, self).setUp()
@@ -422,6 +440,66 @@ class ContextFormatterTestCase(LogTestBase):
                                                message)
         self.assertEqual(expected, self.stream.getvalue())
 
+    def test_user_identity_logging(self):
+        self.config(logging_context_format_string="HAS CONTEXT "
+                                                  "[%(request_id)s "
+                                                  "%(user_identity)s]: "
+                                                  "%(message)s")
+        ctxt = _fake_context()
+        ctxt.request_id = u'99'
+        message = 'test'
+        self.log.info(message, context=ctxt)
+        expected = ("HAS CONTEXT [%s %s %s %s %s %s]: %s\n" %
+                    (ctxt.request_id, ctxt.user, ctxt.tenant, ctxt.domain,
+                     ctxt.user_domain, ctxt.project_domain,
+                     six.text_type(message)))
+        self.assertEqual(expected, self.stream.getvalue())
+
+    def test_user_identity_logging_set_format(self):
+        self.config(logging_context_format_string="HAS CONTEXT "
+                                                  "[%(request_id)s "
+                                                  "%(user_identity)s]: "
+                                                  "%(message)s",
+                    logging_user_identity_format="%(user)s "
+                                                 "%(tenant)s")
+        ctxt = _fake_context()
+        ctxt.request_id = u'99'
+        message = 'test'
+        self.log.info(message, context=ctxt)
+        expected = ("HAS CONTEXT [%s %s %s]: %s\n" %
+                    (ctxt.request_id, ctxt.user, ctxt.tenant,
+                     six.text_type(message)))
+        self.assertEqual(expected, self.stream.getvalue())
+
+    @mock.patch("datetime.datetime",
+                get_fake_datetime(
+                    datetime.datetime(2015, 12, 16, 13, 54, 26, 517893)))
+    @mock.patch("dateutil.tz.tzlocal", new=mock.Mock(return_value=tz.tzutc()))
+    def test_rfc5424_isotime_format(self):
+        self.config(logging_default_format_string="%(isotime)s %(message)s")
+
+        message = "test"
+        expected = "2015-12-16T13:54:26.517893+00:00 %s\n" % message
+
+        self.log.info(message)
+
+        self.assertEqual(expected, self.stream.getvalue())
+
+    @mock.patch("datetime.datetime",
+                get_fake_datetime(
+                    datetime.datetime(2015, 12, 16, 13, 54, 26)))
+    @mock.patch("time.time", new=mock.Mock(return_value=1450274066.000000))
+    @mock.patch("dateutil.tz.tzlocal", new=mock.Mock(return_value=tz.tzutc()))
+    def test_rfc5424_isotime_format_no_microseconds(self):
+        self.config(logging_default_format_string="%(isotime)s %(message)s")
+
+        message = "test"
+        expected = "2015-12-16T13:54:26.000000+00:00 %s\n" % message
+
+        self.log.info(message)
+
+        self.assertEqual(expected, self.stream.getvalue())
+
 
 class ExceptionLoggingTestCase(LogTestBase):
     """Test that Exceptions are logged."""
@@ -447,6 +525,32 @@ class ExceptionLoggingTestCase(LogTestBase):
         log.setup(self.CONF, "test_excepthook_installed")
         self.assertTrue(sys.excepthook != sys.__excepthook__)
 
+    @mock.patch("datetime.datetime",
+                get_fake_datetime(
+                    datetime.datetime(2015, 12, 16, 13, 54, 26, 517893)))
+    @mock.patch("dateutil.tz.tzlocal", new=mock.Mock(return_value=tz.tzutc()))
+    def test_rfc5424_isotime_format(self):
+        self.config(
+            logging_default_format_string="%(isotime)s %(message)s",
+            logging_exception_prefix="%(isotime)s ",
+        )
+
+        product_name = 'somename'
+        exc_log = log.getLogger(product_name)
+
+        self._add_handler_with_cleanup(exc_log)
+        excepthook = log._create_logging_excepthook(product_name)
+
+        try:
+            raise Exception('Some error happened')
+        except Exception:
+            excepthook(*sys.exc_info())
+
+        expected_string = ("2015-12-16T13:54:26.517893+00:00 "
+                           "Exception: Some error happened")
+        self.assertIn(expected_string,
+                      self.stream.getvalue())
+
 
 class FancyRecordTestCase(LogTestBase):
     """Test how we handle fancy record keys that are not in the
@@ -592,9 +696,6 @@ class DomainTestCase(LogTestBase):
 
     def test_domain_in_log_msg(self):
         ctxt = _fake_context()
-        ctxt.domain = 'mydomain'
-        ctxt.project_domain = 'myprojectdomain'
-        ctxt.user_domain = 'myuserdomain'
         user_identity = ctxt.to_dict()['user_identity']
         self.assertTrue(ctxt.domain in user_identity)
         self.assertTrue(ctxt.project_domain in user_identity)
@@ -634,6 +735,10 @@ class SetDefaultsTestCase(BaseTestCase):
         self.assertEqual(_options.DEFAULT_LOG_LEVELS,
                          self.conf.default_log_levels)
 
+    def test_default_log_level_method(self):
+        self.assertEqual(_options.DEFAULT_LOG_LEVELS,
+                         log.get_default_log_levels())
+
     def test_change_default(self):
         my_default = '%(asctime)s %(levelname)s %(name)s [%(request_id)s '\
                      '%(user_id)s %(project)s] %(instance)s'\
@@ -657,7 +762,55 @@ class SetDefaultsTestCase(BaseTestCase):
     def test_log_file_defaults_to_none(self):
         log.set_defaults()
         self.conf([])
-        self.assertEqual(None, self.conf.log_file)
+        self.assertIsNone(self.conf.log_file)
+
+
+@testtools.skipIf(platform.system() != 'Linux',
+                  'pyinotify library works on Linux platform only.')
+class FastWatchedFileHandlerTestCase(BaseTestCase):
+
+    def setUp(self):
+        super(FastWatchedFileHandlerTestCase, self).setUp()
+
+    def _config(self):
+        os_level, log_path = tempfile.mkstemp()
+        log_dir_path = os.path.dirname(log_path)
+        log_file_path = os.path.basename(log_path)
+        self.CONF(['--log-dir', log_dir_path, '--log-file', log_file_path])
+        self.config(use_stderr=False)
+        self.config(watch_log_file=True)
+        log.setup(self.CONF, 'test', 'test')
+        return log_path
+
+    def test_instantiate(self):
+        self._config()
+        logger = log._loggers[None].logger
+        self.assertEqual(1, len(logger.handlers))
+        from oslo_log import watchers
+        self.assertIsInstance(logger.handlers[0],
+                              watchers.FastWatchedFileHandler)
+
+    def test_log(self):
+        log_path = self._config()
+        logger = log._loggers[None].logger
+        text = 'Hello World!'
+        logger.info(text)
+        with open(log_path, 'r') as f:
+            file_content = f.read()
+        self.assertTrue(text in file_content)
+
+    def test_move(self):
+        log_path = self._config()
+        os_level_dst, log_path_dst = tempfile.mkstemp()
+        os.rename(log_path, log_path_dst)
+        time.sleep(2)
+        self.assertTrue(os.path.exists(log_path))
+
+    def test_remove(self):
+        log_path = self._config()
+        os.remove(log_path)
+        time.sleep(2)
+        self.assertTrue(os.path.exists(log_path))
 
 
 class LogConfigOptsTestCase(BaseTestCase):
@@ -672,7 +825,7 @@ class LogConfigOptsTestCase(BaseTestCase):
         self.assertTrue('debug' in f.getvalue())
         self.assertTrue('verbose' in f.getvalue())
         self.assertTrue('log-config' in f.getvalue())
-        self.assertTrue('log-format' in f.getvalue())
+        self.assertTrue('watch-log-file' in f.getvalue())
 
     def test_debug_verbose(self):
         self.CONF(['--debug', '--verbose'])
@@ -686,13 +839,11 @@ class LogConfigOptsTestCase(BaseTestCase):
         self.assertIsNone(self.CONF.log_config_append)
         self.assertIsNone(self.CONF.log_file)
         self.assertIsNone(self.CONF.log_dir)
-        self.assertIsNone(self.CONF.log_format)
 
         self.assertEqual(self.CONF.log_date_format,
                          _options._DEFAULT_LOG_DATE_FORMAT)
 
         self.assertEqual(self.CONF.use_syslog, False)
-        self.assertEqual(self.CONF.use_syslog_rfc_format, True)
 
     def test_log_file(self):
         log_file = '/some/path/foo-bar.log'
@@ -742,14 +893,6 @@ class LogConfigOptsTestCase(BaseTestCase):
         self.CONF(['--logdir', logdir])
         self.assertEqual(self.CONF.log_dir, logdir)
 
-    def test_log_format_overrides_formatter(self):
-        self.CONF(['--log-format', '[Any format]'])
-        log._setup_logging_from_conf(self.CONF, 'test', 'test')
-        logger = log._loggers[None].logger
-        for handler in logger.handlers:
-            formatter = handler.formatter
-            self.assertTrue(isinstance(formatter, logging.Formatter))
-
     def test_default_formatter(self):
         log._setup_logging_from_conf(self.CONF, 'test', 'test')
         logger = log._loggers[None].logger
@@ -758,6 +901,16 @@ class LogConfigOptsTestCase(BaseTestCase):
             self.assertTrue(isinstance(formatter,
                                        formatters.ContextFormatter))
 
+    def test_handlers_cleanup(self):
+        """Test that all old handlers get removed from log_root."""
+        old_handlers = [log.handlers.ColorHandler(),
+                        log.handlers.ColorHandler()]
+        log._loggers[None].logger.handlers = list(old_handlers)
+        log._setup_logging_from_conf(self.CONF, 'test', 'test')
+        handlers = log._loggers[None].logger.handlers
+        self.assertEqual(1, len(handlers))
+        self.assertNotIn(handlers[0], old_handlers)
+
 
 class LogConfigTestCase(BaseTestCase):
 
